@@ -1,7 +1,45 @@
-from home_assistant_bluetooth import BluetoothServiceInfo
+from uuid import UUID
+
+from bleak.backends.device import BLEDevice
+from bluetooth_data_tools import monotonic_time_coarse
+from habluetooth import BluetoothServiceInfo, BluetoothServiceInfoBleak
 
 from bluetooth_sensor_state_data import BluetoothData
 
+
+def make_bluetooth_service_info(  # noqa: PLR0913
+    name: str,
+    manufacturer_data: dict[int, bytes],
+    service_uuids: list[str],
+    address: str,
+    rssi: int,
+    service_data: dict[UUID, bytes],
+    source: str,
+    tx_power: int = 0,
+    raw: bytes | None = None,
+) -> BluetoothServiceInfoBleak:
+    return BluetoothServiceInfoBleak(
+        name=name,
+        manufacturer_data=manufacturer_data,
+        service_uuids=service_uuids,
+        address=address,
+        rssi=rssi,
+        service_data=service_data,
+        source=source,
+        device=BLEDevice(
+            name=name,
+            address=address,
+            details={},
+            rssi=rssi,
+        ),
+        time=monotonic_time_coarse(),
+        advertisement=None,
+        connectable=True,
+        tx_power=tx_power,
+        raw=raw,
+    )
+
+
 SERVICE_INFO_1 = BluetoothServiceInfo(
     name="SensorPush HTP.xw 241",
     address="61DE521B-F0BF-9F44-64D4-75BBE1738105",
@@ -733,4 +771,58 @@ def test_changed_manufacturer_data_with_exclude():
     assert second_data == {1: b"\x00\x00,\x11\x00\x00m\xd3\x11\x01\x12\x01"}
 
     third_data = data.changed_manufacturer_data(SERVICE_INFO_WITH_EXCLUDE_3, {2})
-    assert third_data == {}
+    assert third_data == {1: b"\x00\x00,\x11\x00\x00m\xd3\x11\x01\x12\x01"}
+
+
+def test_changed_manufacturer_data_raw():
+    service_info = make_bluetooth_service_info(
+        name="SensorPush HT.w 0CA1",
+        manufacturer_data={1: b""},  # anything here
+        service_data={},
+        service_uuids=["ef090000-11d6-42ba-93b8-9dd7ec090ab0"],
+        address="aa:bb:cc:dd:ee:ff",
+        rssi=-60,
+        source="local",
+        raw=b"\x06\xff\x04\x9a\xc9\xa5\x46",
+    )
+
+    data = BluetoothData()
+    assert data.changed_manufacturer_data(service_info, {2}) == {39428: b"\xc9\xa5F"}
+    assert data.changed_manufacturer_data(service_info, {39428}) == {1: b""}
+    assert data.changed_manufacturer_data(service_info) == {39428: b"\xc9\xa5F"}
+
+
+def test_changed_manufacturer_data_raw_multiple():
+    service_info = make_bluetooth_service_info(
+        name="SensorPush HT.w 0CA1",
+        manufacturer_data={1: b"", 2: b""},  # anything here
+        service_data={},
+        service_uuids=["ef090000-11d6-42ba-93b8-9dd7ec090ab0"],
+        address="aa:bb:cc:dd:ee:ff",
+        rssi=-60,
+        source="local",
+        raw=b"\x06\xff\x04\x9a\xc9\xa5\x46",
+    )
+
+    data = BluetoothData()
+    assert data.changed_manufacturer_data(service_info, {2}) == {39428: b"\xc9\xa5F"}
+    assert data.changed_manufacturer_data(service_info, {39428}) == {}
+    assert data.changed_manufacturer_data(service_info) == {39428: b"\xc9\xa5F"}
+
+
+def test_single_changed_manufacturer_data_raw():
+    service_info = make_bluetooth_service_info(
+        name="SensorPush HT.w 0CA1",
+        manufacturer_data={39428: b"\xc9\xa5F"},
+        service_data={},
+        service_uuids=["ef090000-11d6-42ba-93b8-9dd7ec090ab0"],
+        address="aa:bb:cc:dd:ee:ff",
+        rssi=-60,
+        source="local",
+        raw=b"\x06",
+    )
+
+    data = BluetoothData()
+    assert data.changed_manufacturer_data(service_info, {2}) == {39428: b"\xc9\xa5F"}
+    assert data.changed_manufacturer_data(service_info, {39428}) == {}
+    assert data.changed_manufacturer_data(service_info) == {39428: b"\xc9\xa5F"}
